package com.ibm.jps.util;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.gmf.runtime.notation.Node;
import org.eclipse.gmf.runtime.notation.NotationPackage;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Namespace;

import com.ibm.xtools.modeler.ui.UMLModeler;
import com.ibm.xtools.uml.ui.diagram.IUMLDiagramHelper;
import com.ibm.xtools.umlnotation.UMLDiagramKind;

/**
 * Contains utilities to manipulate UML diagrams
 */
public class RSMDiagram {

	/**
	 * Retrieves a specific diagram from the specified namespace.
	 * 
	 * @param inNS The namespace to search
	 * @param inDiagName The name of the diagram
	 * @param inKind The diagram kind (e.g. UMLDiagramKind.CLASS_LITERAL)
	 * @return The found diagram or null
	 */
	public static Diagram findDiagram(Namespace inNS, String inDiagName, UMLDiagramKind inKind) {
		IUMLDiagramHelper dgmHelper = UMLModeler.getUMLDiagramHelper();
		for (Object curDiag_o : dgmHelper.getDiagrams(inNS)) {
			Diagram curDiag = (Diagram) curDiag_o;
			if (curDiag.getName().equals(inDiagName) && dgmHelper.getUmlDiagramKind(curDiag).equals(inKind)) {
				return curDiag;
			}
		}
		return null;
	}
	
	/**
	 * Retrieves in the input diagram the node corresponding to the input UML element. If the diagram contains
	 * several nodes pointing to the same element, only the first one is returned.
	 * 
	 * @param inDiag The diagram to search
	 * @param inElt The UML element that the node is pointing to
	 * @return The node matching the input element
	 */
	public static Node findNode(Diagram inDiag, Element inElt) {
		for (Object curView_o : inDiag.getChildren()) {
			View curView = (View) curView_o;
			if (curView instanceof Node) {
				Node node = (Node) curView;
				EObject obj = node.getElement();
				if (obj != null && obj == inElt) {
					return node;
				}
			}
		}
		return null;
	}
	
	/**
	 * Returns a list of Views contained in the specified container and corresponding to the
	 * input UML element.
	 * 
	 * @param inContainer 
	 * 				The input container
	 * @param inRef 
	 * 				The referenced UML element
	 * @return 
	 * 				The list of Views matching the input UML element
	 */
	@SuppressWarnings("unchecked")
	public static List<View> findViews(EObject inContainer, Element inRef) {
		List<View> list = new ArrayList<View>();
		List<EObject> temp = RSMModel.findElements(inContainer, NotationPackage.eINSTANCE.getView());
		for (EObject curElt:temp) {
			View v = (View) curElt;
			EObject e = v.getElement();
			if (e != null && e == inRef) {
				list.add(v);
			}
		}
		return list;
	}
    
}
